/* 
   Actor.cpp

   SpaceShipChase
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/

#include "Actor.h"

#include "World.h"

Actor::Actor()
  : Object()
{
  _collisionRadius = 1.0;

  // set the keys to a defaul
  _keyUp = SDLK_UP;
  _keyDown = SDLK_DOWN;
  _keyLeft = SDLK_LEFT;
  _keyRight = SDLK_RIGHT;

  // check for joysticks
  _joystick = 0;
  
  _ai = false;
  
  _speedFactor = 1.0;
}

Actor::~Actor()
{
  // close any open joysticks
  //if (_joystick)
  //  SDL_JoystickClose(_joystick);
}

/**
 * update the character
 *  and either handle input
 *  or run AI code
 */
void Actor::update()
{
  if (_ai)
    runAI();
  else
    handleInput();
}

/**
 * handle the input
 */
void Actor::handleInput()
{
  Uint8 *keyState = SDL_GetKeyState (NULL);

  int direction = -1;

  if (_joystick)
  {
    int x, y;
    x = SDL_JoystickGetAxis(_joystick, 0);
    y = SDL_JoystickGetAxis(_joystick, 1);

    if (x > 0)
    {
      if (y < 0)
      	direction = UPRIGHT;
      else if (y > 0)
	     direction = DOWNRIGHT;
      else
	     direction = RIGHT;
    }
    else if (x < 0)
    {
      if (y < 0)
	     direction = UPLEFT;
      else if (y > 0)
	     direction = DOWNLEFT;
      else
	     direction = LEFT;
    }
    else
    {
      if (y < 0)
	     direction = UP;
      else if (y > 0)
	     direction = DOWN;
    }
  }
  if (keyState[_keyUp] && keyState[_keyLeft]) // move up and left at 45
    direction = UPLEFT;
  else if (keyState[_keyUp] && keyState[_keyRight]) // move up and right at 45
    direction = UPRIGHT;
  else if (keyState[_keyUp]) // move up
    direction = UP;
  else if (keyState[_keyDown] && keyState[_keyLeft]) // down left at 45
    direction = DOWNLEFT;
  else if (keyState[_keyDown] && keyState[_keyRight]) // down right at 45
    direction = DOWNRIGHT;
  else if (keyState[_keyDown]) // down
    direction = DOWN;
  else if (keyState[_keyLeft]) // just left
    direction = LEFT;
  else if (keyState[_keyRight]) // just right
    direction = RIGHT;
  
  move(direction);
}

/**
 * move in the specifice direction
 */
void Actor::move(int direction)
{
  double radians = 0.0;
  Camera *camera = Camera::getInstance();
 
  double speed = (camera->getSpeed() / 3.0) * camera->getCurrentFPS();
  speed *= _speedFactor;

  Vector3 rotate = getRotate();

  if (direction != -1)
  {
    switch (direction)
    {
    case DOWNRIGHT:
      radians = 315 * (M_PI / 180.0);
      rotate.setY(45);
      break;
    case DOWNLEFT:
      radians = 225 * (M_PI / 180.0);
      rotate.setY(315);
      break;
    case DOWN:
      radians = 270 * (M_PI / 180.0);
      rotate.setY(0);
      break;
    case UPRIGHT:
      radians = 45 * (M_PI / 180.0);
      rotate.setY(135);
      break;
    case UPLEFT:
      radians = 135 * (M_PI / 180.0);
      rotate.setY(225);
      break;
    case UP:
      radians = 90 * (M_PI / 180.0);
      rotate.setY(180);
      break;
    case RIGHT:
      radians = 0 * (M_PI / 180.0);
      rotate.setY(90);
      break;
    case LEFT:
      radians = 180 * (M_PI / 180.0);
      rotate.setY(270);
      break;
    }

    changeAction(WALK);
    
    // move the character
    double x, z;
    x = cos(radians) * speed;
    z = sin(radians) * speed;
    z = -z; // reverse z
    
    Vector3 v = getTranslate();
    v.setX(v.getX() + x);
    v.setZ(v.getZ() + z);
    // make sure this positions isn't outside the world !
    World *w = World::getInstance();
    
    // get the distance from the center
    //double distance = sqrt((v.getX() * v.getX()) + (v.getZ() * v.getZ()));
    Vector3 initial = getInitialPosition();
    double distance = sqrt(((v.getX() + initial.getX()) * (v.getX() + initial.getX())) + ((v.getZ() + initial.getZ()) * (v.getZ() + initial.getZ())));
    
    if (distance > -w->getCollisionRadius() &&
      	distance < w->getCollisionRadius())
      setTranslate(v);
    
    setRotate(rotate);
  }
  else
    changeAction(STILL);
}

/**
 * set the keys, up down left and right
 */
void Actor::setKeys(int up, int down, int left, int right)
{
  _keyUp = up;
  _keyDown = down;
  _keyLeft = left;
  _keyRight = right;
}

/**
 * run the ai code
 */
void Actor::runAI()
{

}
