/* 
   World.cpp

   SpaceShipChase 
   Copyright (C) 2004 Marcus W Dillavou <line72@users.sf.net>
                  and Amy L Dillavou <gs2luv@users.sf.net>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
*/


#include "World.h"

World::World()
  : Object()
{
  _name = "World";

  // load in the mesh
  loadMesh("mesh/IceCreamWorld.jbm");
  // set the radius of the world
  //  !mwd - this really shouldn't be hard coded !
  _collisionRadius = 6.5;

  // create two new characters and give them an
  //  initial position
  _player1 = new Duke();
  _player1->setInitialPosition(Vector3(-1.0, 0.0, 3.0));

  _ship1 = new SpaceShip();
  _ship1->setInitialPosition(Vector3(-4.0, 4.0, 0.0));
  _ship1->setAI(true);
  _ship1->setAggressiveness(1);
  
  _ship2 = new SpaceShip();
  _ship2->setInitialPosition(Vector3(4.0, 4.0, 0.0));
  _ship2->setAI(true);
  _ship2->setAggressiveness(2);
  
  _ship3 = new SpaceShip();
  _ship3->setInitialPosition(Vector3(0.0, 4.0, 4.0));
  _ship3->setAI(true);
  _ship3->setAggressiveness(3);
  
  /**
  // check for joysticks
  SDL_Joystick *joystick1 = 0, *joystick2 = 0;
  int numJoysticks = SDL_NumJoysticks();
  cout << "found : " << numJoysticks << " joysticks\n";
  if (numJoysticks >=1 )
  {
    SDL_JoystickEventState(SDL_ENABLE);
    joystick1 = SDL_JoystickOpen(0);
    if (joystick1)
      _player1->setJoystick(joystick1);
  }
  if (numJoysticks >= 2)
  {
    SDL_JoystickEventState(SDL_ENABLE);
    joystick2 = SDL_JoystickOpen(1);
    if (joystick2)
      _player2->setJoystick(joystick2);
  }
  **/

  // set the keys of player 2
  _ship1->setKeys(SDLK_w, SDLK_s, SDLK_a, SDLK_d);
  _ship2->setKeys(SDLK_i, SDLK_k, SDLK_j, SDLK_l);
  _ship3->setKeys(SDLK_6, SDLK_y, SDLK_t, SDLK_u);

  _captured = false;

  // reset the game timer
  _gameTimer.reset();  
  
  // puase all the ships so they don't immediatly win!
  _ship1->pause();
  _ship2->pause();
  _ship3->pause();
}

World::~World()
{
  if (_player1)
    delete _player1;
  if (_ship1)
    delete _ship1;
  if (_ship2)
    delete _ship2;
  if (_ship3)
    delete _ship3;
}

/**
 * get an instance of the world
 */
World *World::getInstance()
{
  static World instance;

  return &instance;
}

/**
 * draw the world
 *  and all its objects
 */
void World::draw()
{
  // draw the world
  glDisable(GL_CULL_FACE);
  _mesh->draw();
  glEnable(GL_CULL_FACE);

  // draw the players
  _player1->draw();
  _ship1->draw();
  _ship2->draw();
  _ship3->draw();
}

/**
 * update the world
 *  and all its objects
 *
 * returns 
 * -1 if the game is still going on,
 * 0 if draw
 * 1 if player one won
 * 2 if player two won
 */
int World::update()
{
  if (_gameTimer.update() > 30) // each match last 30 seconds
  {
    return 1; // player 1 wins
  }
  if (_captured)
  {
    cout << "you lasted: " << _gameTimer.update() << " seconds\n";
    return 2; // computer wins 
  }

  // check for collision
  checkCollision();

  // update the players movement
  updatePlayersMovement();

  // return -1 to contine with the game
  return -1;
}

/**
 * check to see if a player
 *  two space ships ran into
 *  each other
 */
void World::checkCollision()
{
  // see if two ships collided
  Vector3 v1, v2, v3;
  v1 = _ship1->getAllTranslate();
  v2 = _ship2->getAllTranslate();
  v3 = _ship3->getAllTranslate();
  
  if (fabs(v1.getX() - v2.getX()) < _ship1->getCollisionRadius() * 1.5 &&
      fabs(v1.getZ() - v2.getZ()) < _ship1->getCollisionRadius() * 1.5)
  {
    _ship1->pause();
    _ship2->pause();
    _ship1->shipCollision();
    _ship2->shipCollision();
  }
  if (fabs(v1.getX() - v3.getX()) < _ship1->getCollisionRadius() * 1.5 &&
      fabs(v1.getZ() - v3.getZ()) < _ship1->getCollisionRadius() * 1.5)
  {
    _ship1->pause();
    _ship3->pause();
    _ship1->shipCollision();
    _ship3->shipCollision();
  }  
  if (fabs(v3.getX() - v2.getX()) < _ship1->getCollisionRadius() * 1.5 &&
      fabs(v3.getZ() - v2.getZ()) < _ship1->getCollisionRadius() * 1.5)
  {
    _ship3->pause();
    _ship2->pause();
    _ship3->shipCollision();
    _ship2->shipCollision();
  }
}

/**
 * check for keyboard/joystick
 *  input for moving the player
 */
void World::updatePlayersMovement()
{
  // check for key presses
  _player1->update();
  _ship1->update();
  _ship2->update();
  _ship3->update();

  // make sure we update the animations !
  _player1->updateAnimation();
  _ship1->updateAnimation();
  _ship2->updateAnimation();
  _ship3->updateAnimation();
}
